﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.DataImporter.Command.View;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Linear Transformation", 1, ColumnGroupState.DoubleColumnSelected, "Change Values",
  Position = 6, OptionsView = typeof(LinearTransformationCommandView))]
  public class LinearTransformationCommand : ColumnGroupCommandWithAffectedColumnsBase {
    private LinearTransformationCommand()
      : base(null, string.Empty, null) {
    }

    public LinearTransformationCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
    }

    [Storable]
    private double slope;
    public double Slope {
      get { return this.slope; }
      set { this.slope = value; }
    }

    [Storable]
    private double offset;
    public double Offset {
      get { return this.offset; }
      set { this.offset = value; }
    }

    public override string Description {
      get { return "Lineare Transformation"; }
    }

    public override void Execute() {
      base.Execute();
      DoubleColumn column;
      foreach (int col in AffectedColumns) {
        if (ColumnGroup.GetColumn(col) is DoubleColumn) {
          column = (DoubleColumn)ColumnGroup.GetColumn(col);
          for (int i = 0; i < column.TotalValuesCount; i++)
            if (column.GetValue(i) != null)
              column.ChangeValue(i, (double)column.GetValue(i) * slope + offset);
        }
      }
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      DoubleColumn column;
      foreach (int col in AffectedColumns) {
        if (ColumnGroup.GetColumn(col) is DoubleColumn) {
          column = (DoubleColumn)ColumnGroup.GetColumn(col);
          for (int i = 0; i < column.TotalValuesCount; i++)
            if (column.GetValue(i) != null)
              column.ChangeValue(i, ((double)column.GetValue(i) - offset) / slope);
        }
      }
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }
  }
}
