﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Xml;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.DataImporter.Command.View;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Search and Replace", 1, ColumnGroupState.ColumnSelected, "Change Values",
  Position = 0, OptionsView = typeof(SearchAndReplaceCommandView))]
  public class SearchAndReplaceCommand : SearchWithMatchOperationCommandBase {
    private Dictionary<Point, IComparable> changedCells;
    private List<int> oldSorteColumnIndices;
    private Dictionary<ColumnBase, SortOrder> oldSortOrder;

    private SearchAndReplaceCommand()
      : base(null, string.Empty, null) {
      changedCells = new Dictionary<Point, IComparable>();
      oldSorteColumnIndices = new List<int>();
      oldSortOrder = new Dictionary<ColumnBase, SortOrder>();
    }

    public SearchAndReplaceCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      changedCells = new Dictionary<Point, IComparable>();
      oldSorteColumnIndices = new List<int>();
      oldSortOrder = new Dictionary<ColumnBase, SortOrder>();
    }

    public override string Description {
      get { return "Search and Replace"; }
    }

    [Storable]
    private string replaceValue;
    public string ReplaceValue {
      get { return this.replaceValue; }
      set { this.replaceValue = string.IsNullOrEmpty(value) ? null : value.Trim(); }
    }

    public override void Execute() {
      base.Execute();
      Point[] affectedCells = this.GetAffectedCells();
      ColumnBase column;
      foreach (Point cell in affectedCells) {
        column = this.ColumnGroup.GetColumn(cell.X);
        changedCells[cell] = column.GetValue(cell.Y);
        column.ChangeValueOrLeaveOldValue(cell.Y, ReplaceValue);
      }

      foreach (ColumnBase col in this.ColumnGroup.Columns)
        oldSortOrder.Add(col, col.SortOrder);

      oldSorteColumnIndices = new List<int>(this.ColumnGroup.SortedColumnIndexes);
      this.ColumnGroup.ResetSorting();

      this.ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      //needed because otherwise unchanged cells with the same replaceValue would also be affected
      ColumnBase column;
      foreach (KeyValuePair<Point, IComparable> cell in changedCells) {
        column = this.ColumnGroup.GetColumn(cell.Key.X);
        column.ChangeValueOrLeaveOldValue(cell.Key.Y, cell.Value);
      }

      this.ColumnGroup.SortedColumnIndexes = this.oldSorteColumnIndices;
      foreach (KeyValuePair<ColumnBase, SortOrder> pair in oldSortOrder)
        pair.Key.SortOrder = pair.Value;

      oldSortOrder.Clear();
      oldSorteColumnIndices.Clear();
      changedCells.Clear();
      this.ColumnGroup.FireChanged();
      ColumnGroup = null;
    }
  }
}
